from fastapi.responses import PlainTextResponse
from jinja2 import Environment, FileSystemLoader


class TemplateEngine:
    def __init__(self, dir_path: str) -> None:
        self.dir_path = dir_path
        self.env = Environment(
            loader=FileSystemLoader(dir_path),
            comment_start_string="{#=%",
            comment_end_string="%=#}",
        )

    def render(self, template_name: str, context: dict) -> str:
        template = self.env.get_template(template_name)
        return PlainTextResponse(template.render(context))



import os
import textwrap
from functools import wraps
from typing import Optional, List
from fastapi import Request
from pydantic import BaseModel, field_validator
from mqtt_rest.plugins.template_engine import TemplateEngine

THIS_DIR = os.path.dirname(__file__)


helper_templates = TemplateEngine(
    os.path.join(
        THIS_DIR,
        "_helper_commands",
    ),
)


def bash_function():
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs) -> str:
            if args or kwargs:
                raise ValueError("The function must not accept any arguments")
            result = func(*args, **kwargs)
            if not isinstance(result, str):
                raise ValueError("The function must return a string")
            return (
                f"\n{func.__name__}() {{" + textwrap.indent(textwrap.dedent(result), "    ") + "}"
            )

        return wrapper

    return decorator


class Command(BaseModel):
    command: str
    package: str | None

    @field_validator("package", mode="before")
    def set_package(cls, v, values):
        return v or values.get("command")


def get_installer(
    request: Request,
    dependencies: Optional[List[Command]] = None,
    description: Optional[str] = None,
):
    context = {"request": request}
    if dependencies:
        context["dependencies"] = dependencies
    if description:
        context["description"] = textwrap.dedent(description)
    return helper_templates.render("install.sh", context)


def get_single_job(request: Request, data_command: str):
    context = {"request": request, "data_command": data_command}
    return helper_templates.render("single_job.sh", context)


def get_multi_job(request: Request, get_job_options: str, run_job: str):
    context = {"request": request, "get_job_options": get_job_options, "run_job": run_job}
    return helper_templates.render("multi_job.sh", context)



import os
import argparse
import uvicorn


def get_args():
    parser = argparse.ArgumentParser(
        prog="mqtt-rest-server",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description="MQTT REST Server\nSee help: https://www.github.com/rahulsrma26/mqtt-rest",
    )
    parser.add_argument(
        "-i",
        "--server-ip",
        type=str,
        default=os.environ.get("SERVER_IP", ""),
        help="Host to bind the server",
    )
    parser.add_argument(
        "-p",
        "--server-port",
        type=int,
        default=os.environ.get("SERVER_PORT", 9000),
        help="Port to bind the server",
    )
    parser.add_argument(
        "-l",
        "--server-log-level",
        type=str,
        default=os.environ.get("SERVER_LOG_LEVEL", "info"),
        help="Log level for the server",
    )
    parser.add_argument(
        "-bi",
        "--broker-ip",
        type=str,
        default=os.environ.get("BROKER_IP", ""),
        help="IP of the MQTT Broker",
    )
    parser.add_argument(
        "-bp",
        "--broker-port",
        type=int,
        default=os.environ.get("BROKER_PORT", 1883),
        help="Port of the MQTT Broker",
    )
    parser.add_argument(
        "-mu",
        "--mqtt-user",
        type=str,
        default=os.environ.get("MQTT_USER", ""),
        help="Username for the MQTT Broker",
    )
    parser.add_argument(
        "-mp",
        "--mqtt-password",
        type=str,
        default=os.environ.get("MQTT_PASSWORD", ""),
        help="Password for the MQTT Broker",
    )
    parser.add_argument(
        "-ml",
        "--mqtt-log",
        action="store_true",
        default=os.environ.get("MQTT_LOG", False),
        help="Enable logger for the MQTT Broker",
    )
    return parser.parse_args()


def main():
    args = get_args()
    for key, value in vars(args).items():
        os.environ[key.upper()] = str(value)

    app = "mqtt_rest.server:app"
    uvicorn.run(app, host=args.server_ip, port=args.server_port, log_level=args.server_log_level)


if __name__ == "__main__":
    main()
